"""
A loop is a sequence of instructions that is continually repeated until a certain condition is met. In the context of agent-based systems, a loop can be used to control the behavior of an agent, such as how many times it interacts with other agents or how long it runs for.

- Code Example:

loop = Loop(
    PlanGeneratorAgent(),
    ExecutionAgent(),
    max_loops=10,
)
"""
