from swarms.marketplace.add_all_swarms import autoswarm, router
from swarms.marketplace.agricultural_optimization import (
    AgricultureOptimizationSwarm,
)

__all__ = [
    "autoswarm",
    "router",
    "AgricultureOptimizationSwarm",
]
